--[[ ---------------------------------------------------------------------------

   WintergraspTimer 1.0.0 by Ivan Llanas

--------------------------------------------------------------------------- ]]--

local WT_NAME           = "WintergraspTimer";
local WT_VERSION        = "1.0.0";
local WT_FOLDER         = "Interface\\AddOns\\"..WT_NAME.."\\";

local WINTERGRASP_LEVEL = 80;
local MIN_ALERT_MINUTES = 1;
local MAX_ALERT_MINUTES = 60;
local MIN_IMAGESET      = 1;
local MAX_IMAGESET      = 6;
local MIN_SCALE         = 0.5;
local MAX_SCALE         = 3.0;

-- WINTERGRASP = "Interface\\Icons\\INV_EssenceOfWintergrasp";
-- ALLIANCE    = "|TInterface\\Calendar\\UI-Calendar-Event-PVP02:16|t "
-- HORDE       = "|TInterface\\Calendar\\UI-Calendar-Event-PVP01:16|t "
-- DISPUTED    = "|TInterface\\Calendar\\UI-Calendar-Event-PVP:16|t "

local _font_file        = WT_FOLDER.."Fonts\\Avqest.ttf";
local _wg_mark_texture  = "Interface\\Icons\\INV_Jewelry_Ring_66";
local _stone_shard_texture = "Interface\\Icons\\INV_Misc_Platnumdisks";

local _alert_sound      = WT_FOLDER.."Sounds\\alert.mp3";
local _ranks_sounds     = WT_FOLDER.."Sounds\\rank%d.wav";
local _ranks_texture    = WT_FOLDER.."Images\\rank%d";
local _battle_texture   = WT_FOLDER.."Images\\%dbattle";
local _alliance_texture = WT_FOLDER.."Images\\%dalliance";
local _horde_texture    = WT_FOLDER.."Images\\%dhorde";
local _unknown_texture  = WT_FOLDER.."Images\\%dunknown";

local _time00_color_RGB = CL.clRedL_RGB;
local _time07_color_RGB = CL.clYellow_RGB;
local _time15_color_RGB = CL.clGreenL_RGB;
local _time15_color     = CL.clGreenL;
local _conmsg_color     = CL.clBlueL;

local ST_UNKNOWN        = 0;
local ST_BATTLE         = 1;
local ST_WAITING        = 2;
local _last_state       = ST_UNKNOWN;
local _last_rank        = -1;
local _already_alerted  = false;
local _should_update_icon = false;

WT = {};

local f_update_frame, 
      f_Main, img_Icon, lb_TimeLeft, lb_SecondsLeft, lb_BattleTime, lb_Rank, lb_Rank2,
      img_IconWGMH, img_IconSKS, lb_IconWGMH, lb_IconSKS;
      
-- -----------------------------------------------------------------------------
function _DecodeTime (seconds)
     
   local h, m, s;
   h = 0
   m = 0
   s = seconds
   if s>=3600 then 
      h = floor (s / 3600)
      s = s % 3600
   end        
   if s>=60 then
      m = floor (s / 60)
      s = s % 60
   end
   return h, m, s;
end
        
-- -----------------------------------------------------------------------------
function WT_SetIcon (fmt)

   img_Icon:SetTexture (string.format (fmt, WT.Options.ImageSet));
end;

-- -----------------------------------------------------------------------------
function WT_InitializeVars ()

   if not WT         then  WT = {}; end;
   if not WT.Options then
      WT.Options            = {};
      WT.Options.Visible    = true;
      WT.Options.Locked     = false;
      WT.Options.X          = 0;
      WT.Options.Y          = 0;
      WT.Options.Alert      = true;
      WT.Options.AlertTime  = 15*60;
      WT.Options.RankAlert  = true;
      WT.Options.ImageSet   = MIN_IMAGESET;
   end
   if not WT.Options.Scale then WT.Options.Scale = 1.0; end;
end
        
-- -----------------------------------------------------------------------------
function WT_SetVisible (visible)

   local msg;
   WT.Options.Visible = visible;
   if WT.Options.Visible then 
      f_Main:Show (); 
      msg = WT_NAME.." visible.";
   else                       
      f_Main:Hide ();
      msg = WT_NAME.." hidden.";
   end;
   Print (_conmsg_color..msg);   
end;
        
-- -----------------------------------------------------------------------------
function WT_SetLock (locked)

   local msg;   
   WT.Options.Locked = locked;
   if WT.Options.Locked then msg = WT_NAME.." locked.";
   else                      msg = WT_NAME.." unlocked.";
   end;
   Print (_conmsg_color..msg);   
end

-- -----------------------------------------------------------------------------
function WT_SetAlert (set)

   WT.Options.Alert = set;
end

-- -----------------------------------------------------------------------------
function WT_SetRankAlert (set)

   WT.Options.RankAlert = set;
end

-- -----------------------------------------------------------------------------
function WT_Alert ()

   PlaySoundFile (_alert_sound);
end

-- -----------------------------------------------------------------------------
function WT_RankAlert (rank)

   PlaySoundFile (string.format (_ranks_sounds, rank));             
end;

-- -----------------------------------------------------------------------------
function WT_UpdateFramePos ()

   f_Main:SetPoint ("CENTER", nil, "CENTER", WT.Options.X, WT.Options.Y);
end

-- -----------------------------------------------------------------------------
function WT_Reset (reset_position, skip_alert)

   if reset_position then   
      WT.Options.Visible = true;
      WT.Options.X = 0;
      WT.Options.Y = 0;
      WT.Options.Scale = 1.0;
      WT.Options.Locked = false;
   end;
   f_Main:ClearAllPoints ()
   WT_UpdateFramePos ()
   f_Main:SetScale (WT.Options.Scale);
   local state, _ = WT_GetCurrentState ();
   WT_BattleStatusChanged (state, skip_alert);
end

-- -----------------------------------------------------------------------------
function WT_SetAlertTime (minutes)

   minutes = tonumber (minutes)
   if not minutes then minutes = -1
   else                minutes = floor(minutes)
   end
   if minutes>=MIN_ALERT_MINUTES and minutes<=MAX_ALERT_MINUTES then
      WT.Options.AlertTime = minutes * 60;
      _already_alerted = false;
   end
 end

-- -----------------------------------------------------------------------------
function WT_SetImageSet (number)

   number = tonumber (number)
   if not number then number = -1
   else               number = floor(number)
   end
   if number>=MIN_IMAGESET and number<=MAX_IMAGESET then
      WT.Options.ImageSet = number;
      WT_Reset (false, true);
   end
 end

-- -----------------------------------------------------------------------------
function WT_SetScale (number)

   number = tonumber (number)
   if not number then number = -1 end
   if number>=MIN_SCALE and number<=MAX_SCALE then
      WT.Options.Scale = number;
      WT_Reset (false, true);
   end
 end

-- -----------------------------------------------------------------------------
function _WT_SetTexture (var)

   img_Icon:SetTexture (WT_FOLDER.."Images\\"..var);
end;

-- -----------------------------------------------------------------------------
function WT_Command (cmd)

   cmd = strlower (cmd)
   local _,_,cmd,var = strfind (cmd,"([%w%p]+)%s*(.*)")
   
   if cmd=="tex" then _WT_SetTexture (var)
   else               WT_Settings_Open ()
   end
end
        
-- -----------------------------------------------------------------------------
function f_Main_OnMouseDown (_, btn)
   
   if btn=="LeftButton" and not WT.Options.Locked then
      _,_,_,WT_XposA,WT_YposA = f_Main:GetPoint ()
      f_Main:StartMoving ()
      f_Main:SetBackdropColor (0.0, 0.0, 0.0, 0.4);
      _,_,_,WT_XposB,WT_YposB = f_Main:GetPoint ()
   elseif btn=="RightButton" then
      WT_Settings_Open ()
   end
end

-- -----------------------------------------------------------------------------
function f_Main_OnMouseUp (_, btn)

   if btn=="LeftButton" and not WT.Options.Locked then
      f_Main:SetBackdropColor (0.0, 0.0, 0.0, 0.0);
      local _,_,_,WT_XposC,WT_YposC=f_Main:GetPoint ()
      f_Main:StopMovingOrSizing ()
      f_Main:ClearAllPoints ()
      WT.Options.X = (WT_XposC-WT_XposB) + WT_XposA
      WT.Options.Y = (WT_YposC-WT_YposB) + WT_YposA
      WT_XposA  = nil
      WT_XposB  = nil
      WT_YposA  = nil
      WT_YposB  = nil
      local x = WT.Options.X-floor (WT.Options.X)
      local y = WT.Options.Y-floor (WT.Options.Y)
      if x>=0.5 then WT.Options.X = ceil  (WT.Options.X)
      else           WT.Options.X = floor (WT.Options.X)
      end
      if y>=0.5 then WT.Options.Y = ceil  (WT.Options.Y)
      else           WT.Options.Y = floor (WT.Options.Y)
      end
      WT_UpdateFramePos ()
   end
end
      
-- -----------------------------------------------------------------------------
function WT_GetCurrentState ()

   local next_wg = GetWintergraspWaitTime ();
   local current_state;
   
   if next_wg then current_state = ST_WAITING
   else            current_state = ST_BATTLE;
   end;
   
   return current_state, next_wg;
end;   

-- -----------------------------------------------------------------------------
local _last_elapsed     = 0.0;
function f_update_frame_OnUpdate (_,elapsed)

   _last_elapsed = _last_elapsed + elapsed;
   if _last_elapsed<0.5 then return; end;
   _last_elapsed = 0;
   
   if _should_update_icon then WT_UpdateWaitIcon (); end;
   local current_state, next_wg = WT_GetCurrentState ();
   
   if current_state~=_last_state then WT_BattleStatusChanged (current_state, false); end;
   _last_state = current_state;
   if current_state==ST_BATTLE then WT_UpdateBuffs  ();
   else                             WT_UpdateTimers (next_wg);
   end;
end

-- -----------------------------------------------------------------------------
function f_Main_OnEvent (_, event, ...)

   if (event=="KNOWN_CURRENCY_TYPES_UPDATE" or event=="CURRENCY_DISPLAY_UPDATE") then
      WT_UpdateCurrency ();      
   elseif event=="VARIABLES_LOADED" then
      f_Main:UnregisterEvent ("VARIABLES_LOADED")        
      WT_InitializeVars ()
      WT_Reset (false, false)
      Print (_conmsg_color..WT_NAME.." "..WT_VERSION)
      Print (_conmsg_color.."   "..SLASH_WINTERGRASPTIMER1..", "..SLASH_WINTERGRASPTIMER2..", "..SLASH_WINTERGRASPTIMER3);   
   end;
end

-- -----------------------------------------------------------------------------
function WT_BattleStatusChanged (state, skip_alert)

   if not skip_alert then _already_alerted = false; end;
   _should_update_icon = false;
   _last_rank          = -1;
   lb_SecondsLeft:Hide ();
   if state==ST_BATTLE then
      lb_TimeLeft:Hide ();
      lb_BattleTime:Hide ();
      lb_Rank:Show ();
      lb_Rank2:Show ();
   else
      lb_TimeLeft:Show ();
      lb_BattleTime:Show ();
      lb_Rank:Hide ();
      lb_Rank2:Hide ();

      WT_UpdateNextWGTime ();
      WT_UpdateWaitIcon ();
   end;
   WT_UpdateCurrency ();
end

-- -----------------------------------------------------------------------------
function WT_UpdateTimers (next_wg)

   if next_wg then
      if WT.Options.Alert and (not _already_alerted) then
         if next_wg<=WT.Options.AlertTime then
            WT_Alert ();
            _already_alerted = true;
         end;
      end;
   end;
   
   if not WT.Options.Visible then return end;
   if next_wg then
   if next_wg<9000 then
      local h, m, s = _DecodeTime (next_wg);
      local s_time, time_color;

      if next_wg>900 then time_color = _time15_color; -- 15 mins
      elseif next_wg>600 then 
         time_color = string.format ("|cff%0.2x%0.2x%0.2x", 
                                     floor ((_time07_color_RGB.r + ((_time15_color_RGB.r - _time07_color_RGB.r) / 300 * (next_wg-600)))*255),
                                     floor ((_time07_color_RGB.g + ((_time15_color_RGB.g - _time07_color_RGB.g) / 300 * (next_wg-600)))*255),
                                     floor ((_time07_color_RGB.b + ((_time15_color_RGB.b - _time07_color_RGB.b) / 300 * (next_wg-600)))*255)
                                    );
      else
         time_color = string.format ("|cff%0.2x%0.2x%0.2x", 
                                     floor ((_time00_color_RGB.r + ((_time07_color_RGB.r - _time00_color_RGB.r) / 600 * next_wg))*255),
                                     floor ((_time00_color_RGB.g + ((_time07_color_RGB.g - _time00_color_RGB.g) / 600 * next_wg))*255),
                                     floor ((_time00_color_RGB.b + ((_time07_color_RGB.b - _time00_color_RGB.b) / 600 * next_wg))*255)
                                    );
      end
      if h>0 and m<10 then m = "0"..m end
      if s<10         then s = "0"..s end
      s_time = m;
      if h>0 then 
         s_time = h..":"..s_time; 
         lb_SecondsLeft:SetText (time_color..s);
         if lb_SecondsLeft:IsVisible () then lb_SecondsLeft:Hide (); end;
      else        
         s_time = s_time;
         lb_SecondsLeft:SetText (time_color..s);
         if not lb_SecondsLeft:IsVisible () then lb_SecondsLeft:Show (); end;
      end;
      lb_TimeLeft:SetText (time_color..s_time);
   end
   end
end

-- -----------------------------------------------------------------------------
function WT_UpdateBuffs ()

   if not WT.Options.Visible then return end;
   local i, current_rank;

   for i = 1, 40 do
      local name, rank, icon, count, _,_,_,_,_,_, spellID = UnitAura ("player", i);
      if name then
      if spellID==37795 or spellID==33280 or spellID==55629 then -- if name=="Recruit" or name=="Corporal" or name=="First Lieutenant" then
         current_rank = 0;
         if     spellID==33280 then current_rank = 1 -- Corporal
         elseif spellID==55629 then current_rank = 2 -- First Lieutenant
         end;
         if current_rank~=_last_rank then 
            img_Icon:SetTexture (string.format (_ranks_texture, current_rank)); -- img_Icon:SetTexture (icon);
            if WT.Options.RankAlert then
               WT_RankAlert (current_rank);
            end;
         end;
         _last_rank = current_rank;

         lb_Rank:SetText (name);
         if count>0 then lb_Rank2:SetText (CL.clWhite..count);
         else            lb_Rank2:SetText ("");
         end;
         return;
      end;
      else
      end;
   end;
   lb_Rank:SetText  ("In battle.");
   lb_Rank2:SetText ("");
   WT_SetIcon (_battle_texture);
end;

-- -----------------------------------------------------------------------------
function WT_UpdateWaitIcon ()

   local i;
   WT_SetIcon (_unknown_texture);
   _should_update_icon = _player_level == WINTERGRASP_LEVEL;
   if GetCurrentMapContinent() and not WorldMapFrame:IsVisible() then
      SetMapZoom(4)
      if GetNumMapLandmarks()>0 then
         for i=1,GetNumMapLandmarks() do
            local _,_,tex = GetMapLandmarkInfo (i)
            if tex and (tex==46 or tex==48 or tex==101) then
               if     tex==46  then WT_SetIcon (_alliance_texture); _should_update_icon = false;
               elseif tex==48  then WT_SetIcon (_horde_texture);    _should_update_icon = false;
               elseif tex==101 then WT_SetIcon (_battle_texture);   _should_update_icon = true;
               end
               return;
            end
         end
      end
   end
end

-- -----------------------------------------------------------------------------
function WT_UpdateNextWGTime ()
   
   local h0, m0 = GetGameTime ();
   local NextWG = GetWintergraspWaitTime ();
   
   if not NextWG then return; end;
   
   local h, m, _ = _DecodeTime (NextWG);

   h0 = h0 + h;
   m0 = m0 + m;
   if m0>=60 then
      h0 = h0 + 1;
      m0 = m0 - 60;      
   end;      
   if h0>=24 then h0 = h0 - 24; end;
   if m0<10 then m0="0"..m0 end
   if h0<10 then h0="0"..h0 end
   lb_BattleTime:SetText (CL.clBlizzard.. h0..":"..m0)
end;

-- -----------------------------------------------------------------------------
function WT_UpdateCurrency ()

   local i;
   --img_IconWGMH:SetTexture (_wg_mark_texture);
   --img_IconSKS:SetTexture  (_stone_shard_texture);
   lb_IconWGMH:SetText ("");
   lb_IconSKS:SetText  ("");
   img_IconWGMH:SetAlpha (0.5);
   img_IconSKS:SetAlpha  (0.5);
   for i=1,GetCurrencyListSize() do
      name, _, _, _, _, count, _, icon, itemID = GetCurrencyListInfo (i);      
      if count>0 then
         if itemID==43228 then img_IconSKS:SetAlpha  (1.0);  lb_IconSKS:SetText  (IntToStrThousands(count)); end;
         if itemID==43589 then img_IconWGMH:SetAlpha (1.0);  lb_IconWGMH:SetText (IntToStrThousands(count)); end;
      end;
   end;
end;

-- -----------------------------------------------------------------------------
-- -----------------------------------------------------------------------------
-- -----------------------------------------------------------------------------
local WT_SettingsFrame;

-- -----------------------------------------------------------------------------
function WT_Settings_Open ()

   InterfaceOptionsFrame_OpenToCategory (WT_SettingsFrame);
end;

-- -----------------------------------------------------------------------------
-- -----------------------------------------------------------------------------
-- -----------------------------------------------------------------------------
function WT_CreateSettingsFrame ()
   
   local name, frame, obj1, obj2, obj3, trackbar;
   
   name  = "WT_SettingsFrame";
   frame = getglobal (name);
   
   if not frame then frame = CreateFrame  ("Frame", name);
                     WT_SettingsFrame = frame;
   end;
   
   frame.name    = WT_NAME;
   frame.okay    = false;
   frame.cancel  = false;
   frame.default = false;
   InterfaceOptions_AddCategory (frame);   
   
   obj1 = frame:CreateFontString (WT_NAME.."_Settings_Title", "OVERLAY", "NumberFontNormalHuge");
   obj1:SetJustifyH ("LEFT");
   obj1:SetPoint    ("TOPLEFT",frame,"TOPLEFT",16,-16);
   obj1:SetText     (WT_NAME);

   obj2 = frame:CreateFontString (WT_NAME.."_Settings_Title", "OVERLAY", "GameFontNormal");
   obj2:SetJustifyH ("LEFT");
   obj2:SetPoint    ("TOPLEFT",obj1,"BOTTOMLEFT",0,-2);
   obj2:SetText     ("Version "..CL.clAqua..WT_VERSION);

   obj1 = CreateFrame ("CheckButton", WT_NAME.."Settings_Visible", frame, "OptionsCheckButtonTemplate");
   obj1:SetPoint    ("TOPLEFT",obj2,"BOTTOMLEFT",16,-16);
   getglobal(obj1:GetName().."Text"):SetText("Visible.");
   obj1:SetScript ("OnClick", WT_VisibleChanged);
     
   obj2 = CreateFrame ("CheckButton", WT_NAME.."Settings_Locked", frame, "OptionsCheckButtonTemplate");
   obj2:SetPoint    ("TOPLEFT",obj1,"BOTTOMLEFT",0,0);
   getglobal(obj2:GetName().."Text"):SetText("Locked.");
   obj2:SetScript ("OnClick", WT_LockedChanged);

   obj1 = CreateFrame ("CheckButton", WT_NAME.."Settings_Alert", frame, "OptionsCheckButtonTemplate");
   obj1:SetPoint    ("TOPLEFT",obj2,"BOTTOMLEFT",0,0);
   getglobal(obj1:GetName().."Text"):SetText("Alert");
   trackbar = WT_CreateTrackBar (WT_NAME.."Settings_AlertTime", frame, obj1, MIN_ALERT_MINUTES, MAX_ALERT_MINUTES, 1, WT_AlertTimeChanged);
   obj1:SetScript ("OnClick", WT_AlertChanged);

   obj2 = CreateFrame ("CheckButton", WT_NAME.."Settings_RankAlert", frame, "OptionsCheckButtonTemplate");
   obj2:SetPoint    ("TOPLEFT",obj1,"BOTTOMLEFT",0,0);
   getglobal(obj2:GetName().."Text"):SetText("Ranks alert");
   obj2:SetScript ("OnClick", WT_RankAlertChanged);

   obj1 = frame:CreateFontString (WT_NAME.."_Settings_ImageSetLabel", "OVERLAY", "GameFontNormal");
   obj1:SetJustifyH ("RIGHT");
   obj1:SetPoint    ("TOP",obj2,"BOTTOM",0,-16);
   obj1:SetPoint    ("RIGHT",trackbar,"LEFT",-16,0);
   obj1:SetText     ("Image set");
   trackbar = WT_CreateTrackBar (WT_NAME.."Settings_ImageSet", frame, obj1, MIN_IMAGESET, MAX_IMAGESET, 1, WT_ImageSetChanged);

   obj1 = frame:CreateTexture (WT_NAME.."Settings_Texture2");
   obj1:SetPoint           ("LEFT",trackbar,"CENTER",0,0);
   obj1:SetPoint           ("TOP",trackbar,"BOTTOM",0,-8);
   obj1:SetWidth           (32);
   obj1:SetHeight          (32);
   obj1:SetTexture         (_stone_shard_texture);
   
   obj2 = frame:CreateTexture (WT_NAME.."Settings_Texture1");
   obj2:SetPoint           ("TOP",obj1,"TOP",0,0);
   obj2:SetPoint           ("RIGHT",obj1,"LEFT",0,0);
   obj2:SetWidth           (32);
   obj2:SetHeight          (32);
   obj2:SetTexture         (_wg_mark_texture);

   obj3 = frame:CreateTexture (WT_NAME.."Settings_Texture3");
   obj3:SetPoint           ("TOP",obj1,"TOP",0,0);
   obj3:SetPoint           ("LEFT",obj1,"RIGHT",0,0);
   obj3:SetWidth           (32);
   obj3:SetHeight          (32);
   obj3:SetTexture         (_wg_mark_texture);

   obj1 = frame:CreateTexture (WT_NAME.."Settings_Texture0");
   obj1:SetPoint           ("TOP",obj2,"TOP",0,0);
   obj1:SetPoint           ("RIGHT",obj2,"LEFT",0,0);
   obj1:SetWidth           (32);
   obj1:SetHeight          (32);
   obj1:SetTexture         (_stone_shard_texture);
      
   obj2 = frame:CreateFontString (WT_NAME.."_Settings_ScaleLabel", "OVERLAY", "GameFontNormal");
   obj2:SetJustifyH ("RIGHT");
   obj2:SetPoint    ("TOP",obj1,"BOTTOM",0,-32);
   obj2:SetPoint    ("RIGHT",trackbar,"LEFT",-16,0);
   obj2:SetText     ("Scale");
   trackbar = WT_CreateTrackBar (WT_NAME.."Settings_Scale", frame, obj2, MIN_SCALE, MAX_SCALE, 0.05, WT_ScaleChanged);
   
   obj1 = CreateFrame ("Button", WT_NAME.."Settings_Visible", frame, "GameMenuButtonTemplate");
   obj1:SetPoint    ("TOP",obj2,"BOTTOM",0,-32);
   obj1:SetPoint    ("RIGHT",frame,"RIGHT",-16,0);
   obj1:SetText     ("Reset");
   obj1:SetScript   ("OnClick", WT_ResetClicked);
   
   frame:SetScript ("OnShow", WT_SettingsFrame_OnShow);
end;

-- -----------------------------------------------------------------------------
function WT_SettingsFrame_OnShow ()
   WT_UpdatingSettings = true;
   
   local obj;

-- WT.Options.Visible
   getglobal(WT_NAME.."Settings_Visible"):SetChecked   (WT.Options.Visible);
-- WT.Options.Locked
   getglobal(WT_NAME.."Settings_Locked"):SetChecked    (WT.Options.Locked);
-- WT.Options.Alert
   getglobal(WT_NAME.."Settings_Alert"):SetChecked     (WT.Options.Alert);
-- WT.Options.AlertTime
   obj = getglobal(WT_NAME.."Settings_AlertTime");
   obj:SetValue (floor (WT.Options.AlertTime/60));
-- WT.Options.RankAlert
   getglobal(WT_NAME.."Settings_RankAlert"):SetChecked (WT.Options.RankAlert);
-- WT.Options.ImageSet
   obj = getglobal(WT_NAME.."Settings_ImageSet");
   obj:SetValue (WT.Options.ImageSet);
-- WT.Options.Scale
   obj = getglobal(WT_NAME.."Settings_Scale");
   obj:SetValue (WT.Options.Scale);

   WT_UpdatingSettings = false;
end;

-- -----------------------------------------------------------------------------
function WT_CreateTrackBar (name, parent, attachedto, value0, value1, step, on_change)

   local trackbar, label;
   
   trackbar = CreateFrame   ("Slider", name, parent, "OptionsSliderTemplate");
   trackbar:SetPoint        ("TOP",   attachedto, "TOP", 0, 0);
   trackbar:SetPoint        ("LEFT",  parent, "LEFT", 160, 0);
   trackbar:SetPoint        ("RIGHT", parent, "RIGHT", -16, 0);   
   trackbar:SetMinMaxValues (value0, value1);
   trackbar:SetValueStep    (step);
   getglobal(trackbar:GetName().."Low"):SetText  (value0);
   getglobal(trackbar:GetName().."High"):SetText (value1);  
   
   label = trackbar:CreateFontString (name.."_Value", "OVERLAY", "GameFontNormal");
   label:SetJustifyH ("CENTER");
   label:SetPoint    ("LEFT",trackbar,"LEFT",0,0);
   label:SetPoint    ("RIGHT",trackbar,"RIGHT",0,0);
   label:SetPoint    ("BOTTOM",trackbar,"TOP",0,0);
   label:SetTextColor(1.0, 1.0, 1.0, 1.0);
   label:SetText     ("?");
   
   trackbar:SetScript ("OnValueChanged", on_change);
   return trackbar;
end;  

-- -----------------------------------------------------------------------------
function WT_VisibleChanged ()

   if not WT_UpdatingSettings then WT_SetVisible (getglobal(WT_NAME.."Settings_Visible"):GetChecked());
   end;
end;

-- -----------------------------------------------------------------------------
function WT_LockedChanged ()

   if not WT_UpdatingSettings then WT_SetLock (getglobal(WT_NAME.."Settings_Locked"):GetChecked());
   end;
end;

-- -----------------------------------------------------------------------------
function WT_AlertChanged ()

   if not WT_UpdatingSettings then WT_SetAlert (getglobal(WT_NAME.."Settings_Alert"):GetChecked());
   end;
end;

-- -----------------------------------------------------------------------------
function WT_RankAlertChanged ()

   if not WT_UpdatingSettings then WT_SetRankAlert (getglobal(WT_NAME.."Settings_RankAlert"):GetChecked());
   end;
end;

-- -----------------------------------------------------------------------------
function WT_AlertTimeChanged ()

   local idx;
   idx = getglobal(WT_NAME.."Settings_AlertTime"):GetValue ();
   getglobal(WT_NAME.."Settings_AlertTime_Value"):SetText (idx.." minutes");
   if not WT_UpdatingSettings then WT_SetAlertTime (idx);
   end;
end

-- -----------------------------------------------------------------------------
function WT_ImageSetChanged ()

   local idx;   
   idx = getglobal(WT_NAME.."Settings_ImageSet"):GetValue ();
   getglobal(WT_NAME.."Settings_ImageSet_Value"):SetText (idx);
   getglobal(WT_NAME.."Settings_Texture0"):SetTexture  (string.format (_unknown_texture,  idx));
   getglobal(WT_NAME.."Settings_Texture1"):SetTexture  (string.format (_battle_texture,   idx));
   getglobal(WT_NAME.."Settings_Texture2"):SetTexture  (string.format (_alliance_texture, idx));
   getglobal(WT_NAME.."Settings_Texture3"):SetTexture  (string.format (_horde_texture,    idx));
   
   if not WT_UpdatingSettings then WT_SetImageSet (idx);
   end;
end;

-- -----------------------------------------------------------------------------
function WT_ScaleChanged () 

   local idx;
   idx = getglobal(WT_NAME.."Settings_Scale"):GetValue ();
   getglobal(WT_NAME.."Settings_Scale_Value"):SetText (string.format ("%0.2f", idx));
   if not WT_UpdatingSettings then WT_SetScale (idx);
   end;
end;

-- -----------------------------------------------------------------------------
function WT_ResetClicked ()

   WT_Reset (true, true);
   WT_SettingsFrame_OnShow ()
end;

-- -----------------------------------------------------------------------------
-- -----------------------------------------------------------------------------
-- -----------------------------------------------------------------------------
function WT_CreateMainFrame ()

   f_update_frame = CreateFrame ("Frame");
  
   f_Main = CreateFrame ("Button");
   f_Main:SetWidth             (128);
   f_Main:SetHeight            (52);
   f_Main:SetFrameStrata       ("MEDIUM");
   f_Main:SetClampedToScreen   (true);
   f_Main:SetMovable           (true);
   f_Main:EnableMouse          (true);
   f_Main:SetBackdrop          ({bgFile="Interface/Tooltips/UI-Tooltip-Background",edgeFile="Interface/Tooltips/UI-Tooltip-Border",tile=true,tileSize=16,edgeSize=8,insets={left=4,right=4,top=4,bottom=4}});
   f_Main:SetBackdropColor       (0.0, 0.0, 0.0, 0.0);
   f_Main:SetBackdropBorderColor (1.0, 1.0, 1.0, 0.0);
   f_Main:RegisterEvent        ("VARIABLES_LOADED");

   img_Icon = f_Main:CreateTexture ();
   img_Icon:SetPoint           ("CENTER",f_Main,"CENTER",0,0);
   img_Icon:SetPoint           ("TOP",f_Main,"TOP",0,-4);
   img_Icon:SetWidth           (32);
   img_Icon:SetHeight          (32);
   
   -- local _base_font = "GameFontNormal";
   local _base_font = "NumberFontNormalHuge";
   
   lb_BattleTime = f_Main:CreateFontString (nil,"OVERLAY",_base_font);
   lb_BattleTime:SetFont       (_font_file,12,"OUTLINE");
   lb_BattleTime:SetJustifyH   ("CENTER");
   lb_BattleTime:SetPoint      ("TOP",img_Icon,"BOTTOM",0,0);
   lb_BattleTime:SetText       ("X");
   
   lb_TimeLeft = f_Main:CreateFontString (nil,"OVERLAY",_base_font);
   lb_TimeLeft:SetFont         (_font_file,20,"OUTLINE");
   lb_TimeLeft:SetJustifyH     ("LEFT");
   lb_TimeLeft:SetPoint        ("LEFT",img_Icon,"RIGHT",2,0);
   lb_TimeLeft:SetText         ("X");

   lb_SecondsLeft = f_Main:CreateFontString (nil,"OVERLAY",_base_font);
   lb_SecondsLeft:SetFont      (_font_file,12,"OUTLINE");
   lb_SecondsLeft:SetJustifyH  ("LEFT");
   lb_SecondsLeft:SetPoint     ("TOPLEFT",lb_TimeLeft,"TOPRIGHT",-2,0);
   lb_SecondsLeft:SetText      ("X");

   lb_Rank = f_Main:CreateFontString (nil,"OVERLAY",_base_font);
   lb_Rank:SetFont             (_font_file,20,"OUTLINE");
   lb_Rank:SetJustifyH         ("LEFT");
   lb_Rank:SetPoint            ("LEFT",img_Icon,"RIGHT",0,0);
   lb_Rank:SetText             ("X");

   lb_Rank2 = f_Main:CreateFontString (nil,"OVERLAY",_base_font);
   lb_Rank2:SetFont            (_font_file,16,"OUTLINE");
   lb_Rank2:SetJustifyH        ("LEFT");
   lb_Rank2:SetPoint           ("TOPLEFT",lb_Rank,"BOTTOMLEFT",0,0);
   lb_Rank2:SetText            ("X");

   img_IconWGMH = f_Main:CreateTexture ();
   img_IconWGMH:SetPoint       ("TOPRIGHT",img_Icon,"TOPLEFT",-6,-2);
   img_IconWGMH:SetWidth       (12);
   img_IconWGMH:SetHeight      (12);
   img_IconWGMH:SetTexture     (_wg_mark_texture);
   
   img_IconSKS = f_Main:CreateTexture ();
   img_IconSKS:SetPoint        ("BOTTOMRIGHT",img_Icon,"BOTTOMLEFT",-6,2);
   img_IconSKS:SetWidth        (12);
   img_IconSKS:SetHeight       (12);
   img_IconSKS:SetTexture      (_stone_shard_texture);
   
   lb_IconWGMH = f_Main:CreateFontString (nil,"OVERLAY",_base_font);
   lb_IconWGMH:SetFont         (_font_file,12,"OUTLINE");
   lb_IconWGMH:SetJustifyH     ("LEFT");
   lb_IconWGMH:SetPoint        ("TOPRIGHT",img_IconWGMH,"TOPLEFT",-2,-1);
   lb_IconWGMH:SetTextColor    (1.0, 1.0, 1.0, 1.0);
   lb_IconWGMH:SetText         ("X");
   
   lb_IconSKS = f_Main:CreateFontString (nil,"OVERLAY",_base_font);
   lb_IconSKS:SetFont          (_font_file,12,"OUTLINE");
   lb_IconSKS:SetJustifyH      ("LEFT");
   lb_IconSKS:SetPoint         ("TOPRIGHT",img_IconSKS,"TOPLEFT",-2,-1);
   lb_IconSKS:SetTextColor     (1.0, 1.0, 1.0, 1.0);
   lb_IconSKS:SetText          ("X");
   
   f_Main:RegisterEvent ("KNOWN_CURRENCY_TYPES_UPDATE");
   f_Main:RegisterEvent ("CURRENCY_DISPLAY_UPDATE");
   f_Main:SetScript ("OnEvent",     f_Main_OnEvent);
   f_Main:SetScript ("OnMouseDown", f_Main_OnMouseDown);
   f_Main:SetScript ("OnMouseUp",   f_Main_OnMouseUp);
   f_update_frame:SetScript ("OnUpdate", f_update_frame_OnUpdate);
end;

-- -----------------------------------------------------------------------------
-- -----------------------------------------------------------------------------
-- -----------------------------------------------------------------------------

   WT_CreateMainFrame     ();   
   WT_CreateSettingsFrame ();

   SlashCmdList["WINTERGRASPTIMER"] = WT_Command;
   SLASH_WINTERGRASPTIMER1     = "/wintergrasptimer";
   SLASH_WINTERGRASPTIMER2     = "/wtimer";
   SLASH_WINTERGRASPTIMER3     = "/wt";
